Package: VariantAnnotation
Type: Package
Title: Annotation of Genetic Variants
Description: Annotate variants, compute amino acid coding changes,
        predict coding outcomes.
Version: 1.58.0
Authors@R: c(
        person("Valerie", "Oberchain", role="aut"),
        person("Martin", "Morgan", role="aut"),
        person("Michael", "Lawrence", role="aut"),
        person("Stephanie", "Gogarten", role="ctb"),
        person("Bioconductor Package Maintainer", role="cre",
                email="maintainer@bioconductor.org"))
License: Artistic-2.0
Depends: R (>= 4.0.0), methods, BiocGenerics (>= 0.37.0),
        MatrixGenerics, Seqinfo, GenomicRanges (>= 1.61.1),
        SummarizedExperiment (>= 1.39.1), Rsamtools (>= 2.25.1)
Imports: utils, DBI, Biobase, S4Vectors (>= 0.27.12), IRanges (>=
        2.23.9), XVector (>= 0.29.2), Biostrings (>= 2.77.2),
        AnnotationDbi (>= 1.27.9), rtracklayer (>= 1.69.1), BSgenome
        (>= 1.77.1), GenomicFeatures (>= 1.61.4), curl
Suggests: GenomeInfoDb, RUnit, AnnotationHub,
        BSgenome.Hsapiens.UCSC.hg19, TxDb.Hsapiens.UCSC.hg19.knownGene,
        SNPlocs.Hsapiens.dbSNP144.GRCh37, SIFT.Hsapiens.dbSNP132,
        SIFT.Hsapiens.dbSNP137, PolyPhen.Hsapiens.dbSNP131, snpStats,
        ggplot2, BiocStyle, knitr, magick, jsonlite, httr, rjsoncons
LinkingTo: S4Vectors, IRanges, XVector, Biostrings, Rhtslib (>= 2.99.0)
SystemRequirements: GNU make
LazyLoad: yes
biocViews: DataImport, Sequencing, SNP, Annotation, Genetics,
        VariantAnnotation
Video:
        https://www.youtube.com/watch?v=Ro0lHQ_J--I&list=UUqaMSQd_h-2EDGsU6WDiX0Q
RoxygenNote: 7.3.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/VariantAnnotation
git_branch: RELEASE_3_23
git_last_commit: 2d95a28
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:22:39 UTC; biocbuild
Author: Valerie Oberchain [aut],
  Martin Morgan [aut],
  Michael Lawrence [aut],
  Stephanie Gogarten [ctb],
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
